
/********************************************
****       Awlib 1.1 - Glass window      ****
**** (c) 2002-2009 - Alessandro Pedretti ****
********************************************/


#include <windows.h>
#include <winuser.h>
#pragma hdrstop

#include "awlib.h"


/**** Local variables ****/

static AW_WNDALPHA *    BegAWnd     = NULL;
static AW_WNDALPHA *    PrecAWnd    = NULL;
static BYTE             AwAlphaMax  = AW_DEF_ALPHAMAX;
static BYTE             AwAlphaMin  = AW_DEF_ALPHAMIN;
static BYTE             AwAlphaOpen = AW_DEF_ALPHAOPEN;
static BOOL             AwEnable    = TRUE;
static HANDLE           MthH        = NULL;
static UINT             AwFadeSpeed = AW_DEF_FADESPEED;
static UINT             AwWindows   = 0;
static void            (__stdcall *MySetAlpha)(HWND, COLORREF, BYTE, DWORD) = NULL;

/**** Local prototypes ****/

static LRESULT CALLBACK AW_CallBack(HWND, UINT, WPARAM, LPARAM);
static void             AW_Fade(AW_WNDALPHA *);
static void             AW_FadeInit(AW_WNDALPHA *, BYTE);
static DWORD WINAPI     AW_FadeThr(AW_WNDALPHA *);
static void             AW_KillThr(AW_WNDALPHA *);
static void             AW_SetAlpha(AW_WNDALPHA *, BYTE);


/**** Extra callback for window process ****/

static LRESULT CALLBACK AW_CallBack(HWND hWnd, UINT Msg, WPARAM Wp, LPARAM Lp)
{
  FARPROC                       Proc;

  register AW_WNDALPHA *        AWnd = BegAWnd;
  register AW_WNDALPHA *        Prec = BegAWnd;

  while((AWnd) && (AWnd -> hWnd != hWnd)) {
    Prec = AWnd;
    AWnd = AWnd -> Ptr;
  } /* End of while */
  Proc = AWnd -> Proc;

  if (AwEnable) {
    switch(Msg) {
    case WM_ACTIVATE:
      switch(LOWORD(Wp)) {
      case WA_ACTIVE:
      case WA_CLICKACTIVE:
        AW_FadeInit(AWnd, AwAlphaMax);
        break;

      case WA_INACTIVE:
        AW_FadeInit(AWnd, AwAlphaMin);
        break;
      } /* End of switch */
      break;

    case WM_DESTROY:
      WaitForSingleObject(MthH, INFINITE);
      if (BegAWnd == AWnd) {
        if (PrecAWnd == BegAWnd) PrecAWnd = AWnd -> Ptr;
        BegAWnd = AWnd -> Ptr;
      } else {
        if (PrecAWnd == AWnd) PrecAWnd = Prec;
        Prec -> Ptr = AWnd -> Ptr;
      }
      --AwWindows;
      ReleaseMutex(MthH);
      if (AWnd -> Thread) WaitForSingleObject(AWnd -> Thread, INFINITE);
      if (!AwWindows) {
        CloseHandle(MthH);
        BegAWnd = NULL;
        MthH    = NULL;
      }
#ifdef WIN64
      SetWindowLongPtr(hWnd, GWLP_WNDPROC, (LONG_PTR)Proc);
#else
      SetWindowLong(hWnd, GWL_WNDPROC , (long)Proc);
#endif
      free(AWnd);
      break;

    case WM_QUIT:
    case WM_CLOSE:
      AW_KillThr(AWnd);
      AWnd -> DestAlpha = AwAlphaOpen;
      AW_Fade(AWnd);
      break;

    case WM_SHOWWINDOW:
      if (Wp) AW_SetAlpha(AWnd, AwAlphaOpen);
      break;
    } /* End of switch */
  }

#ifdef __BORLANDC__
  return CallWindowProc((FARPROC)Proc, hWnd, Msg, Wp, Lp);
#else
  return CallWindowProc((WNDPROC)Proc, hWnd, Msg, Wp, Lp);
#endif
}


/**** Enable the glass function ****/

void AW_EnableAll(BOOL Mode)
{
  AW_WNDALPHA           *AWnd;
  HWND                  hFocus;

  if (Mode != AwEnable) {
    hFocus = GetForegroundWindow();
    for(AWnd = BegAWnd; AWnd; AWnd = AWnd -> Ptr) {
      if (!Mode) {
        AW_FadeInit(AWnd, 255);
      } else {
        SetWindowLong(AWnd -> hWnd, GWL_EXSTYLE,
                      GetWindowLong(AWnd -> hWnd, GWL_EXSTYLE) | WS_EX_LAYERED);
        if (AWnd -> hWnd != hFocus) AW_FadeInit(AWnd, AwAlphaMin);
        else if (AwAlphaMax != 255) AW_FadeInit(AWnd, AwAlphaMax);
      }
    } /* End of for */

    AwEnable = Mode;
  }
}


/**** Fade function ****/

static void AW_Fade(AW_WNDALPHA *AWnd)
{
  BYTE                   Step;
  int                    k;

  if ((k = AwFadeSpeed / AW_DELAY) != 0) {
    Step  = (BYTE)((AWnd -> DestAlpha - AWnd -> CurAlpha) / k);
    while(k--) {
      AW_SetAlpha(AWnd, (BYTE)(AWnd -> CurAlpha + Step));
      Sleep(AW_DELAY);
    }
    if (AWnd -> CurAlpha != AWnd -> DestAlpha)
      AW_SetAlpha(AWnd, AWnd -> DestAlpha);
  } else AW_SetAlpha(AWnd, AWnd -> DestAlpha);
}


/**** Fade the window ****/

static void AW_FadeInit(AW_WNDALPHA *AWnd, BYTE NewAlpha)
{
  DWORD         ThreadID;

  if (NewAlpha != AWnd -> DestAlpha) {
    AWnd -> DestAlpha = NewAlpha;
    AW_KillThr(AWnd);
    AWnd -> Thread = CreateThread(NULL, 512, (LPTHREAD_START_ROUTINE)AW_FadeThr, (LPVOID)AWnd, 0, &ThreadID);
  }
}


/**** Manual fade out ****/

void AW_FadeOut(HWND hWnd)
{
  AW_WNDALPHA           *AWnd;

  if (AwEnable) {
    for(AWnd = BegAWnd; (AWnd) && (AWnd -> hWnd != hWnd); AWnd = AWnd -> Ptr);
    if (AWnd) {
      AW_KillThr(AWnd);
      AWnd -> DestAlpha = AwAlphaOpen;
      AW_Fade(AWnd);
    }
  }
}


/**** Fade thread function ****/

static DWORD WINAPI AW_FadeThr(AW_WNDALPHA *AWnd)
{
  AW_Fade(AWnd);
  CloseHandle(AWnd -> Thread);
  AWnd -> Thread = NULL;
  if (!AwEnable) {
    SetWindowLong(AWnd -> hWnd, GWL_EXSTYLE, GetWindowLong(AWnd -> hWnd, GWL_EXSTYLE)&~WS_EX_LAYERED);
  }

  return 0;
}


/**** Initialize the window ****/

AW_WNDALPHA *AW_InitAlphaWin(HWND hWnd)
{
  AW_WNDALPHA *         AWnd = NULL;

  if (GetVersion() < 0x80000000) {
    if ((AWnd = (AW_WNDALPHA *)calloc(sizeof(AW_WNDALPHA), 1)) != NULL) {
      if (!MthH) MthH = CreateMutex(NULL, FALSE, NULL);
      WaitForSingleObject(MthH, INFINITE);
      if (BegAWnd) {
        PrecAWnd -> Ptr = AWnd;
        PrecAWnd        = AWnd;
      } else PrecAWnd = BegAWnd = AWnd;
      ++AwWindows;
      ReleaseMutex(MthH);
      AWnd -> CurAlpha = AwAlphaMax;
      AWnd -> hWnd     = hWnd;
#ifdef WIN64
      AWnd -> Proc     = (FARPROC)SetWindowLongPtr(hWnd, GWLP_WNDPROC, (LONG_PTR)AW_CallBack);
#else
      AWnd -> Proc     = (FARPROC)SetWindowLong(hWnd, GWL_WNDPROC, (long)AW_CallBack);
#endif
      if (AwEnable)
        SetWindowLong(hWnd, GWL_EXSTYLE, GetWindowLong(hWnd, GWL_EXSTYLE)|WS_EX_LAYERED);
    }
  }

  return AWnd;
}


/**** Kill the thread ****/

static void AW_KillThr(AW_WNDALPHA *AWnd)
{
  if (!AWnd -> Thread) return;

  TerminateThread(AWnd -> Thread, 0);
  CloseHandle(AWnd -> Thread);
  AWnd -> Thread = NULL;
}


/**** Set the alpha of a window ****/

static void AW_SetAlpha(AW_WNDALPHA *Wnd, BYTE Alpha)
{
  HANDLE         h;

  if (!MySetAlpha) {
    if ((h = LoadLibrary("USER32.DLL")) != NULL)
      MySetAlpha = (void (__stdcall *)(HWND, COLORREF, BYTE, DWORD))GetProcAddress(h, "SetLayeredWindowAttributes");
  }
  if ((MySetAlpha) && (Alpha != Wnd -> CurAlpha)) {
    MySetAlpha(Wnd -> hWnd, 0, Alpha, LWA_ALPHA);
    Wnd -> CurAlpha = Alpha;
  }
}


/**** Set the parameters ****/

void AW_SetPar(BYTE AlphaMax, BYTE AlphaMin, BYTE AlphaOpen, UINT FadeSpeed)
{
  register AW_WNDALPHA *        AWnd;
  register HWND                 FocusWnd;

  BOOL                          Upd = ((AwAlphaMax != AlphaMax) || (AwAlphaMin != AlphaMin));

  AwAlphaMax  = AlphaMax;
  AwAlphaMin  = AlphaMin;
  AwAlphaOpen = AlphaOpen;
  AwFadeSpeed = FadeSpeed;

  if ((!AwEnable) || (!Upd)) return;

  /**** Update the windows ****/

  FocusWnd = GetForegroundWindow();
  for(AWnd = BegAWnd; AWnd; AWnd = AWnd -> Ptr) {
    if (AWnd -> hWnd == FocusWnd)
      AW_SetAlpha(AWnd, AlphaMax);
    else
      AW_SetAlpha(AWnd, AlphaMin);
  } /* End of for (AWnd) */
}





